__version__ = '0.1.17'
from luvex.node import Node
from luvex.decisiontree import DecisionTree
from luvex.randomforest import RandomForest
from luvex.knn import KNN
from luvex.logisticregression import Logisticregression
from luvex.Linearregression import Linearregression
from luvex.multilinear import MultipleLinear
from luvex.svm import SVM
from luvex.xgbclasstree import XGBClass_Tree
from luvex.xgbclassifier import XGBClass